###############################################################################################
# $Id: new_game.py,v 1.1.1.1 2002/02/19 10:16:10 slmm Exp $
###############################################################################################

import scenario
import properties
import sys
import organization
import widget
from button import Button
from label import Label
from widget_manager import WidgetManager
from dialog import *
from select_scenario import SelectScenario
from checkbox        import CheckBox
#from setup_players   import SetupPlayers


class NewGame(Dialog):
    """
    This class is used as a dialog for setting up parameters for a new game.
    """

    def __init__ (self):
        "Creates the dialog."
        # load the fonts too
        self.titlefont = pygame.font.Font  ( properties.titlefont, 24 )
        self.smallfont = pygame.font.Font  ( properties.textfont, 14 )

        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # flags used for showing the "Ok" button. The two first ones should be set for the button to
        # be shown, and the third is then set
        self.scenario_ok = 0
        self.ok_shown = 0

        # no scenario data yet
        self.lines = None
        

    def createWidgets (self):
        "Creates all widgets for the dialog."

        # labels
        self.wm.register ( Label (self.titlefont, "Start a new game", (10, 10), color=(255, 0, 0)))
        self.wm.register ( Label (self.smallfont, "Scenario:",        (50, 180), color=(255, 255, 255)))
        self.wm.register ( Label (self.smallfont, "Play as: ",        (50, 220), color=(255, 255, 255)) )

        # create the changeable labels
        self.scenariolabel = Label (self.smallfont, "no scenario selected", (200, 180), color=(255, 255, 0))
        #self.sidelabel     = Label (self.smallfont, "not yet set", (200, 210), color=(255, 255, 0))

       # checkboxes
        self.union = CheckBox ( "Union", self.smallfont,
                                properties.path_dialogs + "butt-radio-set.png",
                                properties.path_dialogs + "butt-radio-unset.png",
                                checked=1, position=(180, 210), color=(255,255,0),
                                callbacks={widget.MOUSEBUTTONUP : self.togglePlayer } )
        self.rebel = CheckBox ( "Rebel", self.smallfont,
                                properties.path_dialogs + "butt-radio-set.png",
                                properties.path_dialogs + "butt-radio-unset.png",
                                checked=0, position=(280, 210), color=(255,255,0),
                                callbacks={widget.MOUSEBUTTONUP : self.togglePlayer } )

        # buttons. We show only the 'Cancel' button here, as the 'Ok' button is shown when all is
        # setup properly
        self.wm.register ( Button ( properties.path_dialogs + "butt-scenario-moff.png",
                                    properties.path_dialogs + "butt-scenario-mover.png",
                                    (162, 650 ), {widget.MOUSEBUTTONUP : self.selectScenario } ) )
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (650, 650 ), {widget.MOUSEBUTTONUP : self.cancel } ) )

        # register the labels and checkboxesfor management
        self.wm.register ( self.scenariolabel )
        self.wm.register ( self.rebel ) 
        self.wm.register ( self.union )
 

    def selectScenario (self, widget, event):
        """Callback triggered when the user clicks the 'Select scenario' button. """

        # create the dialog for selecting the scenario
        dialog = SelectScenario ()
        state = dialog.run ()

        # was the dialog rejected?
        if state == REJECTED:
            # no scenario yet. We leave the 'scenario_ok' flag as it is, as it may have been set
            # earlier to a good value.
            self.scenariolabel.setText ( "no scenario selected" )

        else:
            # get the name and location
            name     = scenario.info.getName ()
            location = scenario.info.getLocation ()

            # as well as the raw data
            self.lines = dialog.getChosenScenario ()
            
            # get the date and create a string
            (year, month, day, hour, minute) = scenario.info.getDate ()
            date = `year` + "." + `month` + "." + `day` + " " + `hour` + ":" + `minute`
            
            # create a label and set it
            self.scenariolabel.setText ( name + ", " + location + ", " + date )
            
            # we now have a valid scenario
            self.scenario_ok = 1
 
        # check wether the 'Ok' button should be shown
        self.checkShowOkButton ()
            
        # repaint the stuff if needed
        self.wm.paint (force=1, clear=1)


    def togglePlayer (self, trigger, event):
        """Callback triggered when one of the checkboxes are clicked. This method makes sure the
        other checkboxes are unchecked."""
        # who triggered the event?
        if trigger == self.union:
            # disable the other button
            self.rebel.setChecked ( 0 )

        else:
            # disable the other button
            self.union.setChecked ( 0 )

        # force a repaint as checkboxes leave stuff behind
        self.wm.paint(1)


    def checkShowOkButton (self):
        """This button checks wether the 'Ok' button should be shown. If both the 'players' and
        'scenario' dialogs have been accepted and are ok this method shows the 'Ok' button. It is
        only shown once."""
        # do we already have a button and are the flags ok?
        if not self.ok_shown and self.scenario_ok:
            # all is ok, show the button
            self.wm.register ( Button ( properties.path_dialogs + "butt-ok-moff.png",
                                        properties.path_dialogs + "butt-ok-mover.png", (406, 650 ),
                                        {widget.MOUSEBUTTONUP : self.ok } ) )

            # now we have shown the button
            self.ok_shown = 1
            
        
    def ok (self, triggered, event):
        """Callback triggered when the user clicks the 'Ok' button. Closes the dialog and returns to
        the main dialog. Performs some general scenario setup."""

        # perform general scenario etup
        self.setupScenario ()

        # we're accepting the dialog
        self.state = ACCEPTED

        return widget.DONE


    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""
        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE


    def setupScenario (self):
        """Performs various scenario setup. When this method is called the scenario is already
        loaded and the players have been set up. Currently does:
        * set visibility of units
        * store id of local player. TODO: use the Player class!
        """

        # store the passed data, first the side we have
        if self.rebel.isChecked ():
           # we're playing as rebel 
           scenario.local_player_id = organization.REBEL
        else:
           # we're playing as union 
           scenario.local_player_id = organization.UNION


        # get all the units
        units = scenario.units

        # loop over all units
        for unit in units.values ():
            # set proper visibility on the unit.
            unit.calcView()


    def getChosenScenario (self):
        """Returns the data for the chosen scenario. The data is returned as a list of string. If no
        scenario had been chosen None is returned."""

        return self.lines
    

#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:



#!/usr/bin/env python2.7

#########################################################################
#                            4st Attack                                 #
#########################################################################
# Created by:                                                           #
# Main programming     - "slm" - Jeroen Vloothuis                       #
# Graphics             - "The Corruptor" -- g@spank-the-monkey.org.uk   #
#########################################################################
# Specail thanks:                                                       #
# Everyone in #pygame and the opensource community in general           #
#########################################################################
# This software is licensed under the GPL - General Public License      #
#########################################################################

import os, sys
import pygame, pygame.font, pygame.image, pygame.mixer, pygame.transform
from pygame.locals import *
from startscreen import *
from inisettings import *
import profile
import string

pygame.init()


# Declaration of the variables
images	= None
screen	= None


def playMusic(datadir):
	if pygame.mixer.get_init():
		pygame.mixer.music.load( os.path.join(datadir, 'music', '4stattack.ogg') )
		pygame.mixer.music.play(-1)

def getOptions(argv):
	opts= {}
	while argv:
		if argv[0][0] == '-':
			opts[argv[0]] = argv[1]
			argv = argv[2:]
		else:
			argv = argv[1:]
	return opts

def setDisplay(resolution, fullscreen):
	if fullscreen == 'yes':
		screen = pygame.display.set_mode(resolution, FULLSCREEN)
	else:
		screen = pygame.display.set_mode(resolution)
	
	pygame.display.set_caption('4st Attack 2')
	pygame.mouse.set_visible(1)
	pygame.display.init()
	
	pygame.display.Info()
	return screen

def loadGraphic(image_name, resolution, themename, datadir):
	image_path = os.path.join(datadir, 'themes', themename, resolution, image_name)
	image = pygame.image.load(image_path).convert_alpha()
	return image

def loadGraphics(file_names, resolution, themename, datadir):
	images = {}
	for key in file_names.keys():
		images[key] = loadGraphic(file_names[key], resolution, themename, datadir)
	return images

def quit():
	pygame.quit()
	os._exit(0)

def main():                    
	options = getOptions(sys.argv)
	
	if options.has_key('-ini'):
		ini_settings = IniSettings(options['-ini'])
	else:
		ini_settings = IniSettings('settings.ini')
	
	settings = ini_settings.settings
	
	datadir = settings['path']['data']
	
	if settings['sound']['music'] == 'yes':
		playMusic(datadir)
	
	resolution = settings['video']['resolution']
	res = string.split(resolution, 'x')
	screen = setDisplay((int(res[0]), int(res[1])), settings['video']['fullscreen'])

	# load all images
	images = {}
	img_files = IniSettings(datadir + '/themes/clean/graphics.ini')
	for key in img_files.settings.keys():
		images[key] = loadGraphics(img_files.settings[key], resolution, "clean", datadir)
		#print "Loading:	", key

	locations = IniSettings(datadir + '/themes/clean/'+resolution+'/locations.ini').settings

	startscreen = StartScreen(screen, images, locations, ini_settings)
	startscreen.run()

if __name__ == '__main__':
	main()



# Creditsscreen
from animator import *


class CreditsScreen:
	def __init__(self, surface, images, locations):
		self.images = images
		self.locations = locations

		background = self.images['credits']['background']
		programming =  self.images['credits']['programming']
		graphics =  self.images['credits']['graphics']
		music =  self.images['credits']['music']
		thanks =  self.images['credits']['thanks']
		slm =  self.images['credits']['slm']
		chakie =  self.images['credits']['chakie']
		korruptor =  self.images['credits']['korruptor']
		greenzebra = self.images['credits']['greenzebra']

		self.animator = Animator(surface, background, 16500)

		# Programming
		self.animator.addObject(AnimatedTurnObject(programming, ((1024-programming.get_width())/2+programming.get_width()/2,200), 0.5, 3000))
		self.animator.addObject(AnimatedMoveObject(programming, ((1024-programming.get_width())/2,200), (-programming.get_width(),200), 1000, 3000))

		# Jeroen
		self.animator.addObject(AnimatedTurnObject(slm, ((1024-slm.get_width())/2+slm.get_width()/2,400), 0.5, 3000, 500))
		self.animator.addObject(AnimatedMoveObject(slm, ((1024-slm.get_width())/2,400), (1024,400), 1000, 3500))

		# Graphics
		self.animator.addObject(AnimatedTurnObject(graphics, ((1024-graphics.get_width())/2+graphics.get_width()/2,200), 0.5, 3000, 4500))
		self.animator.addObject(AnimatedMoveObject(graphics, ((1024-graphics.get_width())/2,200), (-graphics.get_width(),200), 1000, 7500))

		# Gareth
		self.animator.addObject(AnimatedTurnObject(korruptor, ((1024-korruptor.get_width())/2+korruptor.get_width()/2,400), 0.5, 3000, 5000))
		self.animator.addObject(AnimatedMoveObject(korruptor, ((1024-korruptor.get_width())/2,400), (1024,400), 1000, 8000))
		
		# Music
		self.animator.addObject(AnimatedTurnObject(music, ((1024-music.get_width())/2+music.get_width()/2,200), 0.5, 3000, 8000))
		self.animator.addObject(AnimatedMoveObject(music, ((1024-music.get_width())/2,200), (-music.get_width(),200), 1000, 11000))

		# Green zebra
		self.animator.addObject(AnimatedTurnObject(greenzebra, ((1024-greenzebra.get_width())/2+greenzebra.get_width()/2,400), 0.5, 3000, 8500))
		self.animator.addObject(AnimatedMoveObject(greenzebra, ((1024-greenzebra.get_width())/2,400), (1024,400), 1000, 11500))

		# Thanks
		self.animator.addObject(AnimatedTurnObject(thanks, ((1024-thanks.get_width())/2+thanks.get_width()/2,200), 0.5, 3000, 11500))
		self.animator.addObject(AnimatedMoveObject(thanks, ((1024-thanks.get_width())/2,200), (-thanks.get_width(),200), 1000, 14500))

		# Chakie
		self.animator.addObject(AnimatedTurnObject(chakie, ((1024-chakie.get_width())/2+chakie.get_width()/2,400), 0.5, 3000, 12000))
		self.animator.addObject(AnimatedMoveObject(chakie, ((1024-chakie.get_width())/2,400), (1024,400), 1000, 15000))



		#self.animator.addObject(AnimatedFPSTimerObject(0, 0, bigfont, white, background.subsurface(0,0,200,50) ))

	def run(self):
		self.animator.play()
		return



from minmax import *

# This class tries to add some strategic value to the min max algorithme
# this one does this by giving scores for stones based on their connections
class WeightedMinMax(MinMax):

	def score(self, node, player, opponent):
		player_score = 0
		opponent_score = 0

		for pos_x in range(7):
			for pos_y in range(len(node.board.state[pos_x])):
				player_score, opponent_score = self.stone_score(node, player, opponent, (pos_x, pos_y), 1, range(-1,1,1))
		
		return player_score #- opponent_score

	def stone_score(self, node, player, opponent, stone, score, stone_range):
		player_score = 0
		opponent_score = 0
		
		for x in stone_range:
			for y in stone_range:
				if (-1 < (stone[0] + x) < 7):
					if( -1 < (stone[1] + y) < len(node.board.state[stone[0] + x])):
						if node.board.state[stone[0] + x][stone[1] + y] == player:
							player_score +=	self.score_line(node, (stone[1] + y, stone[0] + x), (x,y), player)
						elif node.board.state[stone[0] + x][stone[1] + y] == opponent:
							opponent_score += score
		return player_score, opponent_score

	def score_line(self, node, stone, direction, player):
		score = 0
		for modifier in range(4):
			if(-1 < stone[0]+modifier*direction[0] < 7):
				if( -1 < (stone[1] + modifier*direction[1]) < len(node.board.state[stone[0] + modifier*direction[0]])):
					if node.board.state[stone[0]+modifier*direction[0]][stone[1]+modifier*direction[1]] == player:
						score += modifier
		return score



import gui
import pygame
from quit import *
from game import *
from multiplayerscreen import *
from settingsscreen import *
from gui.dialog import *
from gui.image import *
from players import *

class EndGameDialog(Dialog):
	def __init__(self, surface, images, locations):
		self.images = images
		self.locations = locations
		Dialog.__init__ (self, surface)

	def createWidgets(self):
		# Set the background
		#self.surface.blit(self.images['endgamedialog']['background'],(0,0))

		self.wm.register(Image(self.images['endgamedialog']['message'],
			(int(self.locations['endgamedialog']['message_x']),
                        int(self.locations['endgamedialog']['message_y'])),
		))
		self.wm.register(Button(self.images['endgamedialog']['quit'], self.images['endgamedialog']['quit'],
			(int(self.locations['endgamedialog']['quit_x']),
                        int(self.locations['endgamedialog']['quit_y'])),
			callbacks={widget.MOUSEBUTTONUP : self.quit }
		))
		self.wm.register(Button(self.images['endgamedialog']['keepplaying'], self.images['endgamedialog']['keepplaying'],
                        (int(self.locations['endgamedialog']['keepplaying_x']),
                        int(self.locations['endgamedialog']['keepplaying_y'])),
                        callbacks={widget.MOUSEBUTTONUP : self.keepplaying }
                ))

	def quit(self, trigger, event):
		self.state = "quit"
		return widget.DONE

	def keepplaying(self, trigger, event):
		self.state = "keeplaying"
		return widget.DONE


import os, math, sys
import pygame, pygame.font, pygame.image, pygame.mixer, pygame.transform
from pygame.locals import *

class AnimatedObject:
	def getFrame(time):
		return
	def getSurface(time):
		return
	def getPosition(time):
		return

class AnimatedMoveObject:
	def __init__(self, image, startpos, endpos, playtime, delay=0):
		self.image = image
		self.startpos = startpos
		self.endpos = endpos
		self.playtime = playtime
		self.delta = (endpos[0]-startpos[0])/float(playtime), (endpos[1]-startpos[1])/float(playtime)
		self.delay = delay

	def getPosition(self, time):
		offset = (time-self.delay)

		x = self.startpos[0]+int(self.delta[0]*offset)
		y = self.startpos[1]+int(self.delta[1]*offset)
		return x,y

	def getSurface(self, time):
		return self.image

	def visable(self, time):
		if time < self.delay or time-self.delay > self.playtime: return None
		return 1

class AnimatedAlphaObject:
	def __init__(self, image, position, startalpha, endalpha, playtime, delay=0):
		#self.image = pygame.Surface( (image.get_width(),image.get_height()),pygame.locals.SRCALPHA)
		#self.image.blit(image,(0,0))
		self.image = image.convert()

		self.startalpha = startalpha
		self.endalpha = endalpha
		self.playtime = playtime
		if startalpha < endalpha:
			self.delta = (endalpha - startalpha)/float(playtime)
		else:
			self.delta = (startalpha - endalpha)/float(playtime)
		self.delay = delay
		self.position = position

	def getPosition(self, time):
		return self.position

	def getSurface(self, time):
		offset = (time-self.delay)
		alpha = int((self.delta*offset))
#		print self.image.get_alpha()
		self.image.set_alpha(alpha)
		return self.image


	def visable(self, time):
		if time < self.delay or time-self.delay > self.playtime: return None
		return 1

class AnimatedTurnObject:
	def __init__(self, image, position, number_of_turns, playtime, delay=0):
		self.image = image
		self.playtime = playtime
		self.delta = (number_of_turns*3.14)/float(playtime)
		self.delay = delay
		self.position = position

	def getPosition(self, time):
		offset = (time-self.delay)
		position = (self.position[0]-(math.sin(self.delta*offset)*self.image.get_width())/2, self.position[1])
		return position

	def getSurface(self, time):
		offset = (time-self.delay)

		return pygame.transform.scale(self.image, (math.sin(self.delta*offset)*self.image.get_width(), self.image.get_height()))


	def visable(self, time):
		if time < self.delay or time-self.delay > self.playtime: return None
		return 1

class AnimatedRotateObject:
	def __init__(self, image, position, number_of_turns, playtime, delay=0):
		self.image = image
		self.playtime = playtime
		self.delta = (number_of_turns*360)/float(playtime)
		self.delay = delay
		self.position = position

		self.cache={}
		for angle in range(360):
			self.cache[angle]=pygame.transform.rotozoom(self.image, angle, 1)

	def getPosition(self, time):
		offset = (time-self.delay)
		angle = int(self.delta*offset)%360
		return self.position[0]-self.cache[angle].get_width()/2, self.position[1]-self.cache[angle].get_height()/2

	def getSurface(self, time):
		offset = (time-self.delay)
		#return pygame.transform.rotozoom(self.image, self.delta*offset, 1)
		return self.cache[int(self.delta*offset)%360]


	def visable(self, time):
		if time < self.delay or time-self.delay > self.playtime: return None
		return 1


# Based on wavey by Pete Shinners
class AnimatedWaveObject:
	def __init__(self, x, y, font, message, fontcolor, offset=0.0, speed=0.001, amount=10):
		self.x = x
		self.y = y
		self.base = font.render(message, 0, fontcolor)
		self.steps = range(0, self.base.get_width(), 2)
		self.amount = amount
		self.size = self.base.get_rect().inflate(0, amount).size
		self.offset = offset
		self.speed = speed

	def getSurface(self, time):
		s = pygame.Surface(self.size)
		height = self.size[1]
		self.offset += self.speed * time
		for step in self.steps:
			src = Rect(step, 0, 2, height)
			dst = src.move(0, math.cos(self.offset + step*.02)*self.amount)
			s.blit(self.base, dst, src)
		return s

	def getPosition(self, time):
		return (self.x, self.y)

	def visable(self, time):
		return 1

class AnimatedFPSTimerObject:
	def __init__(self, x, y, font, fontcolor, background, average=0):
		self.x = x
		self.y = y
		self.average = average
		self.background = pygame.Surface((200,50))
		self.background.blit(background,(0,0))
		self.oldtime = 0
		self.font = font
		self.fontcolor = fontcolor

	def getSurface(self, time):
		if (time-self.oldtime):
			fps=1000/(time-self.oldtime)
		else:
			fps = 0
		message = "FPS: "+str(fps)
		surface = pygame.Surface((200,50))
		tekst = self.font.render(message, 0, self.fontcolor)
		surface.blit(self.background,(0,0))
		surface.blit(tekst,(0,0))
		self.oldtime = time
		return surface

	def getPosition(self, time):
		return (self.x, self.y)

	def visable(self, time):
		return 1

class Animator:
	def __init__(self, screen, background, playtime, objects = None):
		self.screen = screen
		self.background = background
		self.playtime = playtime
		self.objects = []
		self.starttime = -1
		self.oldrects = {}

	def addObject(self, object):
		self.objects.append(object)

	def drawFrame(self, time):
		for rect in self.oldrects.values():
			self.screen.blit(self.background.subsurface(rect), rect.topleft)

		rects_to_update = []
		newrects={}

		for object in self.objects:
			if(object.visable(time)):
				image = object.getSurface(time)
				position = object.getPosition(time)
				rect = self.screen.blit(image, position)
				newrects[object]=rect
				if self.oldrects.has_key(object):
					rects_to_update.append(rect.union(self.oldrects[object]))
				else:
					rects_to_update.append(rect)

		for key in (filter(lambda x,l=newrects.keys(): x not in l,self.oldrects.keys())):
			rects_to_update.append(self.oldrects[key])

		pygame.display.update(rects_to_update)
		self.oldrects = newrects

	def play(self):
		self.screen.blit(self.background, (0, 0))
		pygame.display.flip()
		self.starttime = pygame.time.get_ticks()
		time = pygame.time.get_ticks() - self.starttime

		while time < self.playtime:
			time = pygame.time.get_ticks() - self.starttime
			self.drawFrame(time)
		# Reset the clippin
		self.screen.set_clip(((0,0), self.background.get_size()))



try:
	import psyco
except ImportError:
	print "Not using psyco (psyco.sf.net), AI not a maximum speed!"


import rules
from player import *
from random import *
import copy
from board import *

class Node:
	def __init__(self, board, move, player):
		self.board = board
		#self.parent = parent
		self.value = 0
		self.childs = []
		self.move = move
		self.player = player

	def __repr__(self):
		if self.childs > 1:
			number=0
			for child in self.childs:
				number=number+1
			return "Has %s child(s)" % number
		return "STATE"

class MinMax(Player):
	type = 'AI'

	def __init__(self, screen, images, settings, difficulty):
                self.screen = screen
                self.images = images
                self.settings = settings
		self.search_depth = difficulty

	def evaluate(self, node, player, opponent, depth):
		if len(node.childs) > 0:
			list = []
			for child in node.childs:
				list.append(child.value)
			if node.player == player:
				node.value = min(list)
			else:
				node.value = max(list)

                else:
			node.value = self.score(node, player, opponent)  / (depth + 1)

	def score(self, node, player, opponent):
		return int(random() * 100) - 50


	def makeBoard(self, move, board, player):
		temp_board = copy.deepcopy(board)
		temp_board.move(move, player)
	        return temp_board

	def listMoves(self, board, player):
		checkmove = rules.isMoveLegal

		options = []
		for move in range(7):
                        if checkmove(board, move):
                                options.append(move)
		return options

	def statespace(self, node, depth, current_player, player, opponent):
		if rules.isWinner(node.board, opponent):
			#print "a lose up ahead"
                        node.value = -10000 + depth
                        return node
                elif rules.isWinner(node.board, player):
			node.value = 10000 - depth
                        return node

		elif self.listMoves(node.board, 0) < 1:
			self.evaluate(node, player, opponent, depth)
			return node
		elif depth < self.search_depth:
			if current_player==2:
				next_player = 1
			else:
				next_player = 2
			for move in self.listMoves(node.board, current_player):
				node.board.move(move, current_player)

				node.childs.append(self.statespace(Node( node.board, move, player), depth+1, next_player, player, opponent))
				node.board.undomove(move)

		self.evaluate(node, player, opponent, depth)
		return node


	def doMove(self, current_board, player, event):
		board = copy.deepcopy(current_board)

		if player == 1: opponent = 2
		else: opponent = 1

		node = Node(board, 0, player)

		node =  self.statespace( node, 0, player, player, opponent);

		bestscore = -100000
		best_moves = []
		#print "New Round"
		for child in node.childs:
			#print "Child value=", child.value, "move=", child.board.last_move
			if child.value >= bestscore:
				#if rules.isMoveLegal(board, child.board.last_move):
				if bestscore == child.value:
					#print "Move added"
					best_moves.append(child.move)
				else:
					#print "New best move"
					bestscore = child.value
					best_moves = [child.move]
		return best_moves[int(random()*len(best_moves))]

	def gameOver(self, move):
		return None
try:
	psyco.bind(MinMax)
	psyco.bind(Node)
except:
	pass



###############################################################################################
# $Id: image.py,v 1.1.1.1 2002/02/19 10:16:01 slmm Exp $
###############################################################################################

from widget import Widget
import widget
import pygame
import pygame.image
from pygame.locals import *

class Image (Widget):
    """
    This class defines a simple image on the screen. No interaction can be had with it. It needs a
    filename from which the surface is loaded.
    """

    # define a shared cache
    cache = {}

    def __init__ (self, image, position = (0,0), callbacks = None):
        """Initializes the widget. Loads the icons from the passed filename."""

        # firts call superclass constructor
        Widget.__init__ (self, position, callbacks)

        self.surface = image
       
#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
